/*
    Copyright (c) 2009 by Joseph Devietti (devietti@cs.washington.edu).

    This file is part of my DMP ASPLOS 2009 Presentation (DAP).

    DAP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DAP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DAP.  If not, see <http://www.gnu.org/licenses/>.
*/ 

﻿package  {
	
	/**
	 * Managing animation timings (durations), specified in numbers of frames.
	 * @author Joe Devietti
	 */
	public class Anim {
		
		// general
		public static const FADE:uint = 15;
		
		// intro
		public static const INTRO_MOVE_CODEBOX:uint = 40;
		public static const INTRO_MOVE_INSN:uint = 40;
		
		// TenK view
		public static const MOVE_TENK_THREADS:uint = 30;
		
		// shtab concept
		public static const MOVE_QUANTUM:uint = 40;
		
		// quantum builders
		public static const SPLIT_QUANTUM:uint = 12;
		public static const SLIDE_CODE:uint = 30;
		
		// execution stuff
		public static const INSN_APPEAR:uint = 15;
		public static const MOVE_TOKEN:uint = 30;
		public static const HILITE_MOVE:uint = 20;
		public static const HILITE_FADE:uint = 20;
		public static const TIME_ARROW_WIPE:uint = 30;
		public static const COMM_ARROW_WIPE:uint = 40;
		public static const INSN_HILITE_MOVE:uint = 20;
		public static const CPU_STATUS_FADE:uint = FADE;
		public static const XACT_COMMIT:uint = 30;
		
		private static var g_current_animation_frame:uint = 0;
		
		public static function getCurrentAnimationFrame():uint { return g_current_animation_frame; }
		public static function setCurrentAnimationFrame(f:uint):void { g_current_animation_frame = f; }
		
		/** Get frame number for the start of an animation */
		public static function Start():uint {
			return g_current_animation_frame;
		}
		
		/** Return the stop frame number for an animation of the specified duration. 
		 * Do *not* increment the global animation frame counter. */
		public static function StopNoIncr(duration:uint):uint {
			return g_current_animation_frame + duration;
		}
		
		/** Return the stop frame number for an animation of the specified duration. 
		 * Increments the global animation frame counter. */
		public static function StopIncr(duration:uint):uint {
			var f:uint = g_current_animation_frame + duration;
			g_current_animation_frame += duration;
			return f;
		}
	
	}
	
}