/*
    Copyright (c) 2009 by Joseph Devietti (devietti@cs.washington.edu).

    This file is part of my DMP ASPLOS 2009 Presentation (DAP).

    DAP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DAP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DAP.  If not, see <http://www.gnu.org/licenses/>.
*/ 

﻿package {
	
	import com.boostworthy.animation.easing.*;
	import com.boostworthy.animation.sequence.tweens.*;
	import com.boostworthy.geom.Path;
	import flash.display.*;
	import flash.geom.Rectangle;
	
	/**
	 * Constants to help with layout of execution animations.
	 * @author Joe Devietti
	 */
	public class ExecLayout {
	
	public static const STAGE_WIDTH:uint = 610;
	public static const COLUMN_WIDTH:Number = STAGE_WIDTH / 3.0;
	private static const COLUMN_FUDGE:int = 15;
	public static const COLUMN_START:Array = [ 0 - COLUMN_FUDGE, COLUMN_WIDTH, (2*COLUMN_WIDTH) + COLUMN_FUDGE ];
	public static const COLUMN_MIDPOINT_START:Array = [ 0, COLUMN_WIDTH - (COLUMN_FUDGE/2), (2*COLUMN_WIDTH) + (COLUMN_FUDGE/2) ];
	
	public static const INSN_HIGHLIGHT_DIM:Number = 0.15;
	public static const ALPHA_DIM:Number = 0.15;
	
	/** fudge factor to get token to overlap cpu box */
	public static const TOKEN_CPU_FUDGE:uint = 8;
	
	public static const INSNS:Array = [ ["store A","store P"], ["store B","load A"], ["store Q","store B"] ];
	public static const NUM_CPUS:uint = INSNS.length;
	public static const INSNS_PER_CPU:uint = INSNS[0].length;
	
	public static const CODE_WIDTH:uint = MyShapes.INSN_WIDTH;
	/** Need enough room for insn highlighter */
	public static const CODE_TOP:uint = 1;
	public static const CODE_GAP:Number = (COLUMN_WIDTH - CODE_WIDTH) / 2;
	
	public static const CPU_TOP:uint = CODE_TOP + (MyShapes.INSN_HEIGHT * INSNS_PER_CPU) + 10;
	public static const CPU_GAP:Number = (COLUMN_WIDTH - MyShapes.CPU_WIDTH) / 2;
	public static const SHTAB_CPU_GAP:Number = CPU_GAP - 30;
	public static var USE_SHTAB_CPU_MODE:Boolean = false;
	
	public static const INSN_TOP:uint = CPU_TOP + MyShapes.CPU_WIDTH + 10;
	public static const INSN_GAP:Number = (COLUMN_WIDTH - MyShapes.INSN_WIDTH) / 2;
	
	public static const XACT_TOP:uint = INSN_TOP;
	public static const XACT_INSN_TOP:uint = INSN_TOP + MyShapes.XACT_YOFFSET;
	public static const XACT_GAP:Number = (COLUMN_WIDTH - MyShapes.XACT_WIDTH) / 2;
	
	public static const COMM_ARROW_COLOR:uint = 0x0;
	public static const COMM_ARROW_XOFFSET:int = 10;
	
	// TOKEN STUFF
	
	/** Track the processor that currently has the token. */
	public static var g_current_token_cpu:uint = 9999;
	
	/** Return x,y coords to position the token at desired processor. */
	public static function tokenAt(proc:uint):Object {
		if ( proc > 2 ) { throw new Error("Invalid proc: "+proc); }
		var xcoord:int;
		if ( USE_SHTAB_CPU_MODE ) {
			xcoord = COLUMN_START[proc] + SHTAB_CPU_GAP - (2*MyShapes.TOKEN_RADIUS) + TOKEN_CPU_FUDGE;
		} else {
			xcoord = COLUMN_START[proc] + CPU_GAP - (2*MyShapes.TOKEN_RADIUS) + TOKEN_CPU_FUDGE;
		}
		return { x:xcoord, y:CPU_TOP - MyShapes.TOKEN_RADIUS }
	}
	/** Compute a nice curved path to move the token from the source processor to the destination processor. */
	private static function tokenPathTo(src_proc:uint, target_proc:uint):Path {
		var p:Path = new Path();
		var src:Object = tokenAt( src_proc );
		p.moveTo( src.x, src.y );
		var target:Object = tokenAt( target_proc );
		p.curveTo( ((src.x + target.x) / 2.0), target.y - 40/*curve up*/, target.x, target.y );
		return p;
	}
	/** Returns a PathTween that moves the token to the desired processor. This is a blocking animation, ie Anim.StopIncr() */
	public static function moveTokenTo(token:Sprite, target_proc:uint):PathTween {
		var last_token_cpu:uint = g_current_token_cpu;
		g_current_token_cpu = target_proc;
		return new PathTween(token, ExecLayout.tokenPathTo(last_token_cpu, target_proc), false, 
							Anim.Start(), Anim.StopIncr(Anim.MOVE_TOKEN), Transitions.CUBIC_IN_AND_OUT);
	}

	
	/** returns coords needed to move hiliter around the given insn (0-indexed) of the specified cpu */
	public static function hiliteInsnCoords(proc_num:uint, insn_num:uint):Object {
		return {x:ExecLayout.COLUMN_START[proc_num] + ExecLayout.CODE_GAP,
		y:ExecLayout.CODE_TOP + (insn_num * MyShapes.INSN_HEIGHT) };
	}
	
	public static function fadeToNoIncr(d:DisplayObject, new_alpha:Number):Tween {
		return new Tween( d, "alpha", new_alpha, Anim.Start(), Anim.StopNoIncr(Anim.FADE), Transitions.LINEAR);
	}
	public static function fadeToIncr(d:DisplayObject, new_alpha:Number):Tween {
		return new Tween( d, "alpha", new_alpha, Anim.Start(), Anim.StopIncr(Anim.FADE), Transitions.LINEAR);
	}

	/** Create a PathTween object that wipes from the top left to the bottom right 
	 * of the given DisplayObjectContainer.  Uses a blocking animation.
	 * @param d the DisplayObjectContainer to be wiped 
	 * @param duration the duration of the wipe, in frames (blocking)
	 * @param transition the name of the transition function to use
	 * @return a PathTween to be added to the timeline */
	public static function pathWipeTLtoBR(d:DisplayObjectContainer,duration:int,transition:String=Transitions.LINEAR):PathTween {
		var path:Path = new Path();
		var bounds:Rectangle = d.getBounds( d );
		path.moveTo( bounds.left, bounds.top );
		path.lineTo( bounds.right, bounds.bottom );
		return new PathTween( Wiper.pathWipe(d,path), path, true, Anim.Start(),
			Anim.StopIncr(duration), transition );
	}
	/** Same as pathWipeTLtoBR, but start at bottom left and wipe to top right.
	 * @see pathWipeTLtoBR */
	public static function pathWipeBLtoTR(d:DisplayObjectContainer,duration:int,transition:String=Transitions.LINEAR):PathTween {
		var path:Path = new Path();
		var bounds:Rectangle = d.getBounds( d );
		path.moveTo( bounds.left, bounds.bottom );
		path.lineTo( bounds.right, bounds.top );
		return new PathTween( Wiper.pathWipe(d,path), path, true, Anim.Start(),
			Anim.StopIncr(duration), transition );
	}
	
	/** Find a sprite in an array by its name. 
	 * @return the first matching Sprite */
	public static function findByName(sprites:Array,name:String):Sprite {
		for each ( var s:Sprite in sprites ) {
			if ( s.name == name ) {
				return s;
			}
		}
		throw new Error( "No element with name="+name );
	}
	
	/** Find a DispayObject in an array by its name. 
	 * @return the first matching DisplayObject */
	public static function findByNameDO(dos:Array,name:String):DisplayObject {
		for each ( var d:DisplayObject in dos ) {
			if ( d.name == name ) {
				return d;
			}
		}
		throw new Error( "No element with name="+name );
	}
	
	/** Remove all sprites with the specified names from the given array of sprites. 
	 * @returns the filtered array */
	public static function removeByName(names:Array,sprites:Array,removefun:Function):Array {
		return sprites.filter( function(s:DisplayObject,i:int,arr:Array):Boolean {
			var found:Boolean = names.some( function(n:String,i:int,a:Array):Boolean{return s.name == n;} );
			if ( found ) {
				removefun.apply( null, [s] );
				return false;
			}
			return true;
		} );	
	}
	
	public static function coords(dobj:DisplayObject):String {
		return ("x:"+dobj.x + " y:"+dobj.y + " width:"+dobj.width + " height:"+dobj.height);
	}
	
}}